#!/bin/sh
#----------------------------------------------------------------------
# Licensed materials - Property of IBM
#
# 5655-FIN
# 5724-D96
#
# (C) Copyright IBM Corp. 2002, 2009  All Rights Reserved.
#
# US Government Users Restricted Rights - Use, duplication
# or disclosure restricted by GSA ADP Schedule Contract
# with IBM Corp.
#----------------------------------------------------------------------
# Version: 1.8
#----------------------------------------------------------------------
#
# NAME:     dnfczmlc
#
# FUNCTION: This script generates CLI command files to
#           migrate configuration entities and roles
#           using the dnicli
#           - to be executed either by the Configuration Administrator
#             ( e.g. SA1, SA2)
#           - or to be executed by the Security Administrator
#             ( e.g. UA1, UA2)
#
#----------------------------------------------------------------------
# print commands as they are executed
# set -x
#----------------------------------------------------------------------
# Declarations
#----------------------------------------------------------------------
LANG=C;
integer rc=0;
integer rc_max=0;
integer j;
integer ParmCount=0;
integer AnzParm;
integer cliCount=0;

# save the count of input parameters
AnzParm=$#;

#----------------------------------------------------------------------
# callheader
#----------------------------------------------------------------------
callheader()
{
  touch $DnfLog;
  rc=$?
  if [ $rc -ne 0 ]
  then
    echo "DnfError: Log file could not be created.";
    echo "DnfInfo: Ensure that you have write permission and enough space available in the current directory.";
    exit -1
  fi
  Datum=$(date +"%D %T");
  echo "DnfInfo: --------------- Program = $PgmName ---------------" \
  | tee -a $DnfLog;
  echo "DnfInfo: $Datum User=$(whoami) Host=$(hostname) Dir=$(pwd)" >>$DnfLog ;
  echo "DnfInfo: Called with following parameters: Instance=$Instance Dual=$Dual" >>$DnfLog ;
  echo "DNFInfo: $command" >>$DnfLog ;
  echo " " >> $DnfLog;
  touch $DnfInfoLog;
  echo "DnfInfo: --------------- Program = $PgmName ---------------" >> $DnfInfoLog;
  echo "DnfInfo: $Datum User=$(whoami) Host=$(hostname) Dir=$(pwd)" >> $DnfInfoLog ;
  echo "DnfInfo: Called with following parameters: Instance=$Instance Dual=$Dual" >>$DnfInfoLog ;
  echo "DNFInfo: $command" >>$DnfInfoLog ;
  echo " " >> $DnfInfoLog;
}

#----------------------------------------------------------------------
# callfooter
#----------------------------------------------------------------------
callfooter()
{
  if [ $rc -gt $rc_max ]
  then
    rc_max=$rc;
  fi;
  echo " " >> $DnfLog ;
  echo "DnfInfo: ---- Last Returncode: $rc --- Max Returncode: $rc_max ----" >>$DnfLog;
  echo " " >> $DnfLog ;
  if [ $rc_max -gt 1 ]
  then
     rm dnfczmlc_*;
     echo "DnfError: Script $0 failed: Max Returncode was $rc_max.";
     echo "DnfError: Script $0 failed: Max Returncode was $rc_max." >> $DnfLog;
     echo "DnfInfo: Check the log file $DnfLog for details.";

  else
     echo "DnfInfo: Script $0 completed successfully.";
     echo "DnfInfo: Script $0 completed successfully." >> $DnfLog;
  fi;
}

#----------------------------------------------------------------------
# logmessage
#----------------------------------------------------------------------
# puts message into the log with header and footer
logmessage()
{
  echo "$DnfLogSeverity: $DnfLogMsg" >> $DnfLog;
  if [ "$DnfLogSeverity" = "DnfError" ]
  then
    echo "$DnfLogSeverity: $DnfLogMsg";
    echo "$DnfLogSeverity: Last return code is $rc." >> $DnfLog;
  else
    if [ "$DnfLogSeverity" = "DnfWarning" ]
    then
      echo "$DnfLogSeverity: Last return code is $rc." >> $DnfLog;
      echo "$DnfLogSeverity: $DnfLogMsg";
    fi
  fi

}

#----------------------------------------------------------------------
# check_timeout
#----------------------------------------------------------------------
check_timeout()
{ 
  error=0;
  case $ParmValue in
    [0-9]|[0-9]*[0-9])
        if [ $ParmValue -lt 20000 ] || [ $ParmValue -gt 9999999 ]
        then
          error=1;
        fi
        ;;
    *)
      error=1;
      ;;
  esac
  if [ $error -eq 1 ]
  then 
    echo "DnfError: Integer value expected, Please input between 20000 to 9999999.";
    echo "DnfError: Wrong Parameter: $i $1";
    # exit program
    callusage;
    exit 1;
  fi
}

#----------------------------------------------------------------------
# check_yes_no
#----------------------------------------------------------------------
check_yes_no()
{
  if [ "$ParmValue" != "YES" ] && [ "$ParmValue" != "NO" ]
  then
    echo "DnfError: YES/NO expected, Wrong Parameter: $i $1";
    # exit program
    callusage;
    exit 1;
  fi;
}


#----------------------------------------------------------------------
# catFiles
#----------------------------------------------------------------------
catFiles()
{
  touch $CliEntAll;
  rm $CliEntAll;
  touch $CliEntAll;

  touch $tempCliRemCOSCom;
  cat $tempCliRemCOSCom >> $CliEntAll;
  rm $tempCliRemCOSCom;
  touch $tempCliRemCOSDep;
  cat $tempCliRemCOSDep >> $CliEntAll;
  rm $tempCliRemCOSDep;

  touch $tempCliRemCOCom;
  cat $tempCliRemCOCom >> $CliEntAll;
  rm $tempCliRemCOCom;
  touch $tempCliRemCODep;
  cat $tempCliRemCODep >> $CliEntAll;
  rm $tempCliRemCODep;

  touch $tempCliRemCTCom;
  cat $tempCliRemCTCom >> $CliEntAll;
  rm $tempCliRemCTCom;
  touch $tempCliRemCTDep;
  cat $tempCliRemCTDep >> $CliEntAll;
  rm $tempCliRemCTDep;

  touch $tempCliCreCTCom;
  cat $tempCliCreCTCom >> $CliEntAll;
  rm $tempCliCreCTCom;
  touch $tempCliCreCTDep;
  cat $tempCliCreCTDep >> $CliEntAll;
  rm $tempCliCreCTDep;

  touch $tempCliCreCOCom;
  cat $tempCliCreCOCom >> $CliEntAll;
  rm $tempCliCreCOCom;
  touch $tempCliCreCODep;
  cat $tempCliCreCODep >> $CliEntAll;
  rm $tempCliCreCODep;

  touch $tempCliCreCOSCom;
  cat $tempCliCreCOSCom >> $CliEntAll;
  rm $tempCliCreCOSCom;
  touch $tempCliCreCOSDep;
  cat $tempCliCreCOSDep >> $CliEntAll;
  rm $tempCliCreCOSDep;

  fileToCheck="$tempCliValCom";
  checkFileAccess;
  rcval=$?
  if [ $rcval -eq 0 ]
  then
    touch $CliValAll;
    rm $CliValAll;
    touch $CliValAll;

    touch $tempCliValCom;
    cat $tempCliValCom >> $CliValAll;
    rm $tempCliValCom;
    touch $tempCliValDep;
    cat $tempCliValDep >> $CliValAll;
    rm $tempCliValDep;
  fi

  fileToCheck="$tempCliRemRoCom";
  checkFileAccess;
  rcremro=$?
  if [ $rcremro -eq 0 ]
  then
    touch $CliRemRoAll;
    rm $CliRemRoAll;
    touch $CliRemRoAll;

    touch $tempCliRemRoCom;
    cat $tempCliRemRoCom >> $CliRemRoAll;
    rm $tempCliRemRoCom;
    touch $tempCliRemRoApp;
    cat $tempCliRemRoApp >> $CliRemRoAll;
    rm $tempCliRemRoApp;
  fi

  fileToCheck="$tempCliCreRoCom";
  checkFileAccess;
  rccrero=$?
  if [ $rccrero -eq 0 ]
  then
    touch $CliCreRoAll;
    rm $CliCreRoAll;
    touch $CliCreRoAll;

    touch $tempCliCreRoCom;
    cat $tempCliCreRoCom >> $CliCreRoAll;
    rm $tempCliCreRoCom;
    touch $tempCliCreRoApp;
    cat $tempCliCreRoApp >> $CliCreRoAll;
    rm $tempCliCreRoApp;
  fi
}

#----------------------------------------------------------------------
# callcheckparm
#----------------------------------------------------------------------
# Check the script input parameters
callcheckparm()
{
  # Help required
  if [ "$1" = "-?" ]
  then
    rc=0;
    # exit program
    callusage;
    exit 1;
  fi

  # Evaluate the modulo 2 value
  j=$AnzParm%2;
  # If count of parameters is an odd number then callusage
  if [ $j -ne 0 ]
  then
   echo "DnfError: Missing Parameter or value.";
    callusage;
    # exit program
    exit 1;
  fi;

  # Set the default for dual authorization
  Dual="YES";
  Timeout=100000;

  all="";
  for i in $*
  do
    # Get the value to the parameter (shift parameter one to left)
    shift

    # Evaluate the modulo 2 value
    j=$ParmCount%2;
    # Skip the values, process only parameters
    if [ $j -eq 0 ]
    then
#     echo "$i $1"
      ParmValue=$1;
      case $i in
        -i    ) Instance=$1 ;;
        -dual ) check_yes_no; Dual=$1 ;;
        -to   ) check_timeout; Timeout=$1;;
        *     )
              echo "DnfError: Invalid parameter: $i";
              callusage;
              # exit program
              exit 1;;
      esac;
    fi;
    ParmCount=ParmCount+1;

  done;
  # end of reading parameters

  if [ "$Instance" = "" ]
  then
     echo "DnfError: Instance not set.";
     callusage;
     # exit program
     exit 1;
  fi;

}

#----------------------------------------------------------------------
# callusage
#----------------------------------------------------------------------
callusage()
{
     echo "This script creates CLI command files. These command files contain the required statements"
     echo "to migrate WebSphere BI for FN configuration and security entities."
     echo "Usage: Call ..."
     echo "$PgmName "
     echo "         -i    <instance>"
     echo "         -dual YES|NO"
     echo "  -i    is mandatory, no default"
     echo "  -dual is optional, YES / NO, default=YES, case sensitive"
     echo "  -to   is optional, The timeout interval that the CLI waits for responses, default=100000"
     echo "e.g. "
     echo "$PgmName -i INST1 -dual YES -to 100000"

   exit 1;
}

#----------------------------------------------------------------------
# callplatform
#----------------------------------------------------------------------
# Get the platform value AIX or OS/390
# and set the rexx program name

callplatform()
{
  # Get platform
  platform=`uname`
  if [ "$platform" != "$DNI_USS" ] && [ "$platform" != "$DNI_AIX" ]
  then
    echo "DnfError: Unsupported platform: $platform";
    # Exit the program
    exit 1;
  fi;
}


# ============================================================================
# Check file Access
# ============================================================================
# Checks whether the file is accesible
# if yes 0 is returned; 1 otherwise
# input fileToCheck      (the file to be checked)
# input DnfLog           (name of the log file)

checkFileAccess()
{

  echo "" |
  awk -v logFile="$DnfLog" -v fileToCheck="$fileToCheck" '
  BEGIN {
    rc=0;
    rc1 = getline text <fileToCheck;
    #printf "rc1=%d for file %s\n", rc1, fileToCheck;
    if (rc1 == -1) rc=1;
  }
  {
  }
  END {
    exit rc;
  }
  ';
}


# ============================================================================
# Check path for trailing slash
# ============================================================================
# Checks whether the path has a trailing slash
# if yes 0 is returned; 1 otherwise
# input pathToCheck (the path to be checked)
# input DnfLog           (name of the log file)

checkForTrailingSlash()
{
  echo "" |
  awk '
  BEGIN {
    rc=1;
  }
  {
  }
  END {
    pos=0;
    go=1;
    while (go == 1) {
      pos = index(path, "/");
      if (pos > 0) {
        path = substr(path, pos+1);
        if (length(path) == 0) {
          rc=0;
          go=0;
        }
      } else {
        go=0;
      }
    }
    exit rc;
  }
  ' logFile="$DnfLog" path="$pathToCheck";
}

# ============================================================================
# Check existing SVB
# ============================================================================
# Checks if any SVBs installed into WBIFN
# if ok 0 is returned; 16 otherwise
# input Instance         (the instance name)
# input DnfLog           (name of the log file)

checkSVBExists() {

  DnfLogMsg="Checking integrity of the database ...";
  DnfLogSeverity="DNFInfo";
  logmessage;
  dnicli -i "$Instance" -s DNI_SYSADM -ou SYSOU -c "list -ou DNFSYSOU -ct DniConfigSVB -lo o" |
  awk '
    BEGIN {
      FS=" ";
      rc=0;
      svbConfigCnt=0;
      svbfound=0;
    }
    {
      # collect all existing OUs in table
      if ($1 == "DNIP3301I") {
        svbConfigCnt=svbConfigCnt+1;
        printf "DNFInfo: ... SVB %s found.\n", $2 >> logFile;
      }
    }
    END {
      if (svbConfigCnt == 0){
        # error
        rc=16;
        printf "DNFError: No any SVBs defined in WBIFN.\n" >> logFile;
      }
      else {
        printf "DNFInfo: Totally %s SVBs defined in WBIFN.\n", svbConfigCnt >> logFile;
      }
      exit rc;
    }
  ' logFile="$DnfLog";
}

# ============================================================================
# call migfile to generate temporary resource files
# ============================================================================
# Checks if any SVBs installed into WBIFN
# if ok 0 is returned; 16 otherwise
# input Instance         (the instance name)
# input DnfLog           (name of the log file)

generateTempFile() {

  DnfLogMsg="Generate temporary resource file ...";
  DnfLogSeverity="DNFInfo";
  logmessage;
  dnicli -i "$Instance" -s DNI_SYSADM -ou SYSOU -c "list -ou DNFSYSOU -ct DniConfigSVB -lo osv" |
  awk '
    BEGIN {
      FS=" ";
      rc=0;
      MigfileConunt=0;
    }
    {
      if ($1 == "DNIP3301I") {
          #printf "%s\n", $3 >> logFile;
          if ($3 == "migfile"){ 
             svbtab[$2]=$2;
             migfile[$2]=$4;
             if ($2 == "DNFCOMMON"){
                 rundatadir[$2]= "$DNI_PATH";
             }else if (($2 == "DNFFCOMMON") || ($2 == "DNFEFAS")){
                 rundatadir[$2]= "$DNF_PATH";
             }else if (($2 == "DNQCOMMON") || ($2 == "DNQERBS")){
                 rundatadir[$2]= "$DNQ_PATH";
             }else{
                 printf("rundatadir for %s not exist\n",$2);
             }
             #printf("svbtab:%s,migfile:%s\n",svbtab[$2],migfile[$2]);
             #printf("rundatadir:%s\n",rundatadir[$2]);
          }
      }
    }
    END {
      for (x in svbtab){  
      	#printf "svbtab:%s, rundatadir:%s,migfile:%s\n",svbtab[x],rundatadir[svbtab[x]],migfile[svbtab[x]]>>logFile;
      	if ((rundatadir[svbtab[x]] != "") && (migfile[svbtab[x]] != "")) {
          rfile = sprintf("%s",migfile[svbtab[x]]);
          printf "DNFInfo: ... Execute migration file %s for SVB %s.\n", migfile[svbtab[x]], svbtab[x] >> logFile;
          cmd = sprintf("%s %s %s %s %s %s %s\/run\/data\/ %s",rfile, tempctfile, tempcosfile, tempcofile, temprolefile, temphisfile, rundatadir[svbtab[x]], logFile);
          #printf "%s", cmd >> logFile;
          if (system(cmd)!=0){
            rc=1;
            printf "DNFError: ... Error when execute Migration file %s for SVB %s.\n", rfile, svbtab[x] >> logFile; 
          }        
          MigfileCount = MigfileCount + 1;
          printf "DNFInfo: ... Resource file %s successfully added into temporary file.\n", rfile >> logFile;          
        }
        else{
           printf "DNFInfo: ... Configuration Setting for SVB %s not found.\n", svbtab[x] >> logFile;
           rc=1;
        }
      }      
      if (rc>0 || MigfileCount==0){
        # error
        rc=16;
        printf "DNFError: Generate temporary files for migration error.\n" >> logFile;
      }
      else {
        printf "DNFInfo: Totally %s migration file execute successfully.\n", MigfileCount >> logFile;
      }
      exit rc;
    }
  ' tempctfile="$tempctfile" tempcosfile="$tempcosfile" tempcofile="$tempcofile"\
    temprolefile="$temprolefile" temphisfile="$temphisfile" logFile="$DnfLog";
}

# ============================================================================
# Check existing OU
# ============================================================================
# Checks the number of existing OUs
# if ok 0 is returned; 16 otherwise
# input Instance         (the instance name)
# input DnfLog           (name of the log file)
# input ouToCheck        (OU to be checked; "" - basic installation, "%" - whole installation)

checkOUExists()
{
  DnfLogMsg="Checking if $ouToCheck exists ...";
  DnfLogSeverity="DnfInfo";
  logmessage;
  dnicli -i "$Instance" -s DNI_SYSADM -ou SYSOU -c "list -ou % -lo n" |
  awk '
  BEGIN {
    FS=" ";
    rc=0;
    ouExists=0;
  }
  {
    # collect all existing OUs in table
    if ($1 == "DNIP3301I") {
      #printf "x=0, %s %s\n", $1, $2;
      if ($2 == ouToCheck) ouExists=1;
      outab[$2]=1;
    }
  }
  END {
    cnt=0;
    for (x in outab) {
      cnt=cnt+1;
      printf "DnfInfo: ... OU %s exists.\n", x >> logFile;
    }
    if (ouExists == 0) {
      # error
      rc=16;
      printf "DnfError: OU %s does not exist.\n", ouToCheck >> logFile;
      printf "DnfError: OU %s does not exist.\n", ouToCheck;
    }
    exit rc;
  }
  ' logFile="$DnfLog" ouToCheck="$ouToCheck";
}

# ============================================================================
# Check for not approved Roles
# ============================================================================
# Checks wether there are Roles in a not approved state
# if ok 0 is returned; 16 otherwise
# input Instance         (the instance name)
# input DnfLog           (name of the log file)

checkNotApprovedRo()
{
  DnfLogMsg="Checking for not approved Roles ...";
  rc_max=0;
  DnfLogSeverity="DnfInfo";
  logmessage;

  # ===========================================
  # check for not approved Roles with SWIFTNet%
  # ===========================================
  dnicli -i "$Instance" -s DNI_SECADM -ou SYSOU -c "list -ro SWIFTNet% -qo mor -lo n" |
  awk '
  BEGIN {
    FS=" ";
    rc=0;
  }
  {
    # parse for not approved Roles
    if ($1 == "DNIU7050I") {
      printf "DnfError: Role %s not approved. Approve the Role and run the script again.\n", $2 >> logFile;
      printf "DnfError: Role %s not approved. Approve the Role and run the script again.\n", $2;
      rc=16;
    }
  }
  END {
    exit rc;
  }
  ' logFile="$DnfLog";

  rc=$?;

  if [ $rc -gt $rc_max ]
  then
    rc_max=$rc;
    exit $rc_max;
  fi;

  # ===========================================
  # check for not approved Roles with Sag%
  # ===========================================

  dnicli -i "$Instance" -s DNI_SECADM -ou SYSOU -c "list -ro Sag% -qo mor -lo n" |
  awk '
  BEGIN {
    FS=" ";
    rc=0;
  }
  {
    # parse for not approved Roles
    if ($1 == "DNIU7050I") {
      printf "DnfError: Role %s not approved. Approve the Role and run the script again.\n", $2 >> logFile;
      printf "DnfError: Role %s not approved. Approve the Role and run the script again.\n", $2;
      rc=16;
    }
  }
  END {
    exit rc;
  }
  ' logFile="$DnfLog";

  rc=$?;

  if [ $rc -gt $rc_max ]
  then
    rc_max=$rc;
    exit $rc_max;
  fi;

  # ===========================================
  # check for not approved Role Dnf
  # ===========================================

  dnicli -i "$Instance" -s DNI_SECADM -ou SYSOU -c "list -ro Dnf% -qo mor -lo n" |
  awk '
  BEGIN {
    FS=" ";
    rc=0;
  }
  {
    # parse for not approved Roles
    if ($1 == "DNIU7050I") {
      printf "DnfError: Role %s not approved. Approve the Role and run the script again.\n", $2 >> logFile;
      printf "DnfError: Role %s not approved. Approve the Role and run the script again.\n", $2;
      rc=16;
    }
  }
  END {
    exit rc;
  }
  ' logFile="$DnfLog";

  rc=$?;

  if [ $rc -gt $rc_max ]
  then
    rc_max=$rc;
    exit $rc_max;
  fi;
  
  # ===========================================
  # check for not approved Role Dnq
  # ===========================================

  dnicli -i "$Instance" -s DNI_SECADM -ou SYSOU -c "list -ro Dnq% -qo mor -lo n" |
  awk '
  BEGIN {
    FS=" ";
    rc=0;
  }
  {
    # parse for not approved Roles
    if ($1 == "DNIU7050I") {
      printf "DnfError: Role %s not approved. Approve the Role and run the script again.\n", $2 >> logFile;
      printf "DnfError: Role %s not approved. Approve the Role and run the script again.\n", $2;
      rc=16;
    }
  }
  END {
    exit rc;
  }
  ' logFile="$DnfLog";

  rc=$?;

  if [ $rc -gt $rc_max ]
  then
    rc_max=$rc;
    exit $rc_max;
  fi;
}


# ============================================================================
# Check for not deployed CTs
# ============================================================================
# Checks wether there are CTs in a not deployed state
# if ok 0 is returned; 16 otherwise
# input Instance         (the instance name)
# input DnfLog           (name of the log file)

checkNotDeployedCT()
{
  DnfLogMsg="Checking for not deployed CTs ...";
  DnfLogSeverity="DnfInfo";
  logmessage;

  # =====================================
  # check for not deployed CT's with Dnf%
  # =====================================
  dnicli -i "$Instance" -s DNI_SYSADM -ou SYSOU -c "list -ct Dnf% -qo amorz -lo n" |
  awk '
  BEGIN {
    FS=" ";
    rc=0;
  }
  {
    # parse for not deployed CTs
    if ($1 == "DNIP3300I") {
      printf "DnfError: CT %s not deployed. Deploy the CT and run the script again.\n", $2 >> logFile;
      printf "DnfError: CT %s not deployed. Deploy the CT and run the script again.\n", $2;
      rc=16;
    }
  }
  END {
    exit rc;
  }
  ' logFile="$DnfLog";

  rc=$?;

  if [ $rc -gt $rc_max ]
  then
    rc_max=$rc;
    exit $rc_max;
  fi;



  # =====================================
  # check for not deployed CT with DNF%
  # =====================================
  dnicli -i "$Instance" -s DNI_SYSADM -ou SYSOU -c "list -ct DNF% -qo amorz -lo n" |
  awk '
  BEGIN {
    FS=" ";
    rc=0;
  }
  {
    # parse for not deployed CTs
    if ($1 == "DNIP3300I") {
      printf "DnfError: CT %s not deployed. Deploy the CT and run the script again.\n", $2 >> logFile;
      printf "DnfError: CT %s not deployed. Deploy the CT and run the script again.\n", $2;
      rc=16;
    }
  }
  END {
    exit rc;
  }
  ' logFile="$DnfLog";

  rc=$?;

  if [ $rc -gt $rc_max ]
  then
    rc_max=$rc;
    exit $rc_max;
  fi;
  
  # =====================================
  # check for not deployed CT with Dnq%
  # =====================================
  dnicli -i "$Instance" -s DNI_SYSADM -ou SYSOU -c "list -ct Dnq% -qo amorz -lo n" |
  awk '
  BEGIN {
    FS=" ";
    rc=0;
  }
  {
    # parse for not deployed CTs
    if ($1 == "DNIP3300I") {
      printf "DnfError: CT %s not deployed. Deploy the CT and run the script again.\n", $2 >> logFile;
      printf "DnfError: CT %s not deployed. Deploy the CT and run the script again.\n", $2;
      rc=16;
    }
  }
  END {
    exit rc;
  }
  ' logFile="$DnfLog";

  rc=$?;

  if [ $rc -gt $rc_max ]
  then
    rc_max=$rc;
    exit $rc_max;
  fi;
}


# ============================================================================
# Check for not deployed OUs
# ============================================================================
# Checks wether there are OUs in a not deployed state
# if ok 0 is returned; 16 otherwise
# input Instance         (the instance name)
# input DnfLog           (name of the log file)

checkNotDeployedOU()
{
  DnfLogMsg="Checking for not deployed OUs ...";
  DnfLogSeverity="DnfInfo";
  logmessage;
  # =====================================
  # check for not deployed OU's
  # =====================================
  dnicli -i "$Instance" -s DNI_SYSADM -ou SYSOU -c "list -ou % -qo amorz -lo n" |
  awk '
  BEGIN {
    FS=" ";
    rc=0;
  }
  {
    # parse for not deployed OUs
    if ($1 == "DNIP3301I") {
      printf "DnfError: OU %s not deployed. Deploy the OU and run the script again.\n", $2 >> logFile;
      printf "DnfError: OU %s not deployed. Deploy the OU and run the script again.\n", $2;
      rc=16;
    }
  }
  END {
    exit rc;
  }
  ' logFile="$DnfLog";
}

# ============================================================================
# Check for not deployed COSs
# ============================================================================
# Checks wether there are COSs in a not deployed state
# if ok 0 is returned; 16 otherwise
# input Instance         (the instance name)
# input DnfLog           (name of the log file)

checkNotDeployedCOS()
{
  DnfLogMsg="Checking for not deployed COSs ...";
  DnfLogSeverity="DnfInfo";
  logmessage;

  # ======================================
  # check for not deployed COS's with Dnf%
  # ======================================
  dnicli -i "$Instance" -s DNI_SYSADM -ou SYSOU -c "list -cos Dnf% -qo amorz -lo n" |
  awk '
  BEGIN {
    FS=" ";
    rc=0;
  }
  {
    # parse for not deployed COSs
    if ($1 == "DNIP3303I") {
      printf "DnfError: COS %s not deployed. Deploy the COS and run the script again.\n", $2 >> logFile;
      printf "DnfError: COS %s not deployed. Deploy the COS and run the script again.\n", $2;
      rc=16;
    }
  }
  END {
    exit rc;
  }
  ' logFile="$DnfLog";

  rc=$?;

  if [ $rc -gt $rc_max ]
  then
    rc_max=$rc;
    exit $rc_max;
  fi;

  # ======================================
  # check for not deployed COS SAGOPCFG1
  # ======================================
  dnicli -i "$Instance" -s DNI_SYSADM -ou SYSOU -c "list -cos SAGOPCFG1 -qo amorz -lo n" |
  awk '
  BEGIN {
    FS=" ";
    rc=0;
  }
  {
    # parse for not deployed COSs
    if ($1 == "DNIP3303I") {
      printf "DnfError: COS %s not deployed. Deploy the COS and run the script again.\n", $2 >> logFile;
      printf "DnfError: COS %s not deployed. Deploy the COS and run the script again.\n", $2;
      rc=16;
    }
  }
  END {
    exit rc;
  }
  ' logFile="$DnfLog";

  rc=$?;

  if [ $rc -gt $rc_max ]
  then
    rc_max=$rc;
    exit $rc_max;
  fi;
  
  # ======================================
  # check for not deployed COS's with Dnq%
  # ======================================
  dnicli -i "$Instance" -s DNI_SYSADM -ou SYSOU -c "list -cos Dnq% -qo amorz -lo n" |
  awk '
  BEGIN {
    FS=" ";
    rc=0;
  }
  {
    # parse for not deployed COSs
    if ($1 == "DNIP3303I") {
      printf "DnfError: COS %s not deployed. Deploy the COS and run the script again.\n", $2 >> logFile;
      printf "DnfError: COS %s not deployed. Deploy the COS and run the script again.\n", $2;
      rc=16;
    }
  }
  END {
    exit rc;
  }
  ' logFile="$DnfLog";

  rc=$?;

  if [ $rc -gt $rc_max ]
  then
    rc_max=$rc;
    exit $rc_max;
  fi;
}


# ============================================================================
# Check CLI
# ============================================================================
# Checks if the CLI can be accessed properly

checkCli()
{
  DnfLogMsg="Checking CLI for instance $Instance ...";
  DnfLogSeverity="DnfInfo";
  logmessage;

  cmd="dnicli -i $Instance -s DNI_SYSADM -ou SYSOU -to $Timeout -c 'list -cos % -lo nco'"
  dnicli -i "$Instance" -s DNI_SYSADM -ou SYSOU -to $Timeout -c 'list -cos % -lo nco' > dnfczmlc_temp
  rc=$?
  if [ $rc -eq 0 ]
  then
    cmd="dnicli -i $Instance -s DNI_SYSADM -ou SYSOU -to $Timeout -c 'list -ct % -lo nsl'"
    dnicli -i "$Instance" -s DNI_SYSADM -ou SYSOU -to $Timeout -c 'list -ct % -lo nsl' > dnfczmlc_temp
    rc=$?
  fi
  if [ $rc -eq 0 ]
  then
    cmd="dnicli -i $Instance -s DNI_SECADM -ou SYSOU -to $Timeout -c 'list -ro % -lo ncos'"
    dnicli -i "$Instance" -s DNI_SECADM -ou SYSOU -to $Timeout -c 'list -ro % -lo ncos' > dnfczmlc_temp
    rc=$?
  fi
  rm dnfczmlc_temp
  if [ $rc -ne 0 ]
  then
    DnfLogMsg="CLI command does not run properly: $cmd";
    DnfLogSeverity="DnfError";
    logmessage;
    DnfLogMsg="Script cannot run.";
    logmessage;
    DnfLogMsg="Please check the following:";
    logmessage;
    DnfLogMsg="  - Did you let the dniprofile and dnfprofile run?";
    logmessage;
    DnfLogMsg="  - Does the user have all necessary authorizations?";
    logmessage;
    DnfLogMsg="  - Does the instance $instance exist?";
    logmessage;
    DnfLogMsg="  - Is the timeout interval for the CLI high enough?";
    logmessage;
    DnfLogMsg="  - Run the above command in the shell and see";
    logmessage;
    DnfLogMsg="    the error message issued by the CLI";
    logmessage;
    callfooter;
    # Exit the program
    exit 1;
  fi
}


# ============================================================================
# Query CLI
# ============================================================================

queryCLI()
{
  DnfLogMsg="Query on the current configuration ...";
  DnfLogSeverity="DnfInfo";
  logmessage;

  # ==========================================================================================
  # List all ESN Roles (Sag% or SWIFTNet% or Dnf% or Dnq%) with CT's, CO's and attributes
  # and store in dnfczmlc_RoCTCOAttrList
  # ==========================================================================================

  if [ $rc -eq 0 ]
  then
    dnicli -i "$Instance" -s DNI_SECADM -ou SYSOU -to $Timeout -c "list -ro Sag% -lo ncos" > dnfczmlc_RoCTCOAttrList
    rc=$?
  fi
  if [ $rc -eq 0 ]
  then
    dnicli -i "$Instance" -s DNI_SECADM -ou SYSOU -to $Timeout -c "list -ro SWIFTNet% -lo ncos" >> dnfczmlc_RoCTCOAttrList
    rc=$?
  fi
  if [ $rc -eq 0 ]
  then
    dnicli -i "$Instance" -s DNI_SECADM -ou SYSOU -to $Timeout -c "list -ro Dnf% -lo ncos" >> dnfczmlc_RoCTCOAttrList
    rc=$?
  fi
  if [ $rc -eq 0 ]
  then
    dnicli -i "$Instance" -s DNI_SECADM -ou SYSOU -to $Timeout -c "list -ro Dnq% -lo ncos" >> dnfczmlc_RoCTCOAttrList
    rc=$?
  fi
  if [ $rc -eq 0 ]
  then
    dnicli -i "$Instance" -s DNI_SECADM -ou SYSOU -to $Timeout -c "list -ro FT_USER -lo ncos" >> dnfczmlc_RoCTCOAttrList
    rc=$?
  fi

  # ============================================================================
  # List all ESN COS's (Dnf% or SAGOPCFG1 or Dnq%) and store in dnfczmlc_COSList
  # ============================================================================
  DnfLogMsg="Query Extension for SWIFTNet COS's ...";
  DnfLogSeverity="DnfInfo";
  logmessage;
  if [ $rc -eq 0 ]
  then
    dnicli -i "$Instance" -s DNI_SYSADM -ou SYSOU -to $Timeout -c "list -cos Dnf% -lo n" > dnfczmlc_COSList
    rc=$?
  fi
  if [ $rc -eq 0 ]
  then
    dnicli -i "$Instance" -s DNI_SYSADM -ou SYSOU -to $Timeout -c "list -cos SAGOPCFG1 -lo n" >> dnfczmlc_COSList
    rc=$?
  fi
  if [ $rc -eq 0 ]
  then
    dnicli -i "$Instance" -s DNI_SYSADM -ou SYSOU -to $Timeout -c "list -cos Dnq% -lo n" >> dnfczmlc_COSList
    rc=$?
  fi

  # ========================================================================================
  # List all ESN COS's (Dnf% or SAGOPCFG1 or Dnq%) with their Internal Types (Dni%, Dnf%
  # or DNF% or Dnq%) and store in dnfczmlc_COSCTCOList
  # ========================================================================================
  if [ $rc -eq 0 ]
  then
    dnicli -i "$Instance" -s DNI_SYSADM -ou SYSOU -to $Timeout -c "list -cos Dnf% -ct Dnf% -lo nco" > dnfczmlc_COSCTCOList
    rc=$?
  fi
  if [ $rc -eq 0 ]
  then
    dnicli -i "$Instance" -s DNI_SYSADM -ou SYSOU -to $Timeout -c "list -cos SAGOPCFG1 -ct Dnf% -lo nco" >> dnfczmlc_COSCTCOList
    rc=$?
  fi
  if [ $rc -eq 0 ]
  then
    dnicli -i "$Instance" -s DNI_SYSADM -ou SYSOU -to $Timeout -c "list -cos Dnf% -ct DNF% -lo nco" >> dnfczmlc_COSCTCOList
    rc=$?
  fi
  if [ $rc -eq 0 ]
  then
    dnicli -i "$Instance" -s DNI_SYSADM -ou SYSOU -to $Timeout -c "list -cos Dnq% -ct Dnq% -lo nco" >> dnfczmlc_COSCTCOList
    rc=$?
  fi
  if [ $rc -eq 0 ]
  then
    dnicli -i "$Instance" -s DNI_SYSADM -ou SYSOU -to $Timeout -c "list -cos SAGOPCFG1 -ct DNF% -lo nco" >> dnfczmlc_COSCTCOList
    rc=$?
  fi
  if [ $rc -eq 0 ]
  then
    dnicli -i "$Instance" -s DNI_SYSADM -ou SYSOU -to $Timeout -c "list -cos Dnf% -ct Dni% -lo nco" >> dnfczmlc_COSCTCOList
    rc=$?
  fi
  if [ $rc -eq 0 ]
  then
    dnicli -i "$Instance" -s DNI_SYSADM -ou SYSOU -to $Timeout -c "list -cos SAGOPCFG1 -ct Dni% -lo nco" >> dnfczmlc_COSCTCOList
    rc=$?
  fi
  if [ $rc -eq 0 ]
  then
    dnicli -i "$Instance" -s DNI_SYSADM -ou SYSOU -to $Timeout -c "list -cos Dnq% -ct Dni% -lo nco" >> dnfczmlc_COSCTCOList
    rc=$?
  fi


  # ============================================================================
  # List all COS's (%) using ESN Types (Dnf% or DNF% or Dnq%) and store in
  # dnfczmlc_ExtCOSCTCOList
  # ============================================================================
  if [ $rc -eq 0 ]
  then
    dnicli -i "$Instance" -s DNI_SYSADM -ou SYSOU -to $Timeout -c "list -cos Dni% -ct Dnf% -lo nco" > dnfczmlc_ExtCOSCTCOList
    rc=$?
  fi
  if [ $rc -eq 0 ]
  then
    dnicli -i "$Instance" -s DNI_SYSADM -ou SYSOU -to $Timeout -c "list -cos Dni% -ct DNF% -lo nco" >> dnfczmlc_ExtCOSCTCOList
    rc=$?
  fi
  if [ $rc -eq 0 ]
  then
    dnicli -i "$Instance" -s DNI_SYSADM -ou SYSOU -to $Timeout -c "list -cos Dni% -ct Dnq% -lo nco" >> dnfczmlc_ExtCOSCTCOList
    rc=$?
  fi

  # ============================================================================
  # List all CT's (DniConfigSVB, Dnf%, DNF%, Dnq% and DniMsgDomain) and store in dnfczmlc_CTList
  # ============================================================================
  DnfLogMsg="Query Extension for SWIFTNet CT's ...";
  DnfLogSeverity="DnfInfo";
  logmessage;
  if [ $rc -eq 0 ]
  then
    dnicli -i "$Instance" -s DNI_SYSADM -ou SYSOU -to $Timeout -c "list -ct DniConfigSVB% -lo n" > dnfczmlc_CTList
    rc=$?
  fi
  if [ $rc -eq 0 ]
  then
    dnicli -i "$Instance" -s DNI_SYSADM -ou SYSOU -to $Timeout -c "list -ct Dnf% -lo n" >> dnfczmlc_CTList
    rc=$?
  fi
  if [ $rc -eq 0 ]
  then
    dnicli -i "$Instance" -s DNI_SYSADM -ou SYSOU -to $Timeout -c "list -ct DNF% -lo n" >> dnfczmlc_CTList
    rc=$?
  fi
  if [ $rc -eq 0 ]
  then
    dnicli -i "$Instance" -s DNI_SYSADM -ou SYSOU -to $Timeout -c "list -ct Dnq% -lo n" >> dnfczmlc_CTList
    rc=$?
  fi
     # =================================================
     # Speical condition for SR2010 ct DniMsgDomain
     # =================================================
  if [ $rc -eq 0 ]
  then
    dnicli -i "$Instance" -s DNI_SYSADM -ou SYSOU -to $Timeout -c "list -ct DniMsgDomain -lo n" >> dnfczmlc_CTList
    rc=$?
  fi

  # ============================================================================
  # List all CT's (Dnf%, DNF%, Dni% with Attr Dnf%, Dnq%, DniConfigSVB and TestMsgDefSet) with attributes and types and store in
  # dnfczmlc_CTAttrList
  # ============================================================================
  if [ $rc -eq 0 ]
  then
    dnicli -i "$Instance" -s DNI_SYSADM -ou SYSOU -to $Timeout -c "list -ct DniConfigSVB -lo nsl" > dnfczmlc_CTAttrList
    rc=$?
  fi
  if [ $rc -eq 0 ]
  then
    dnicli -i "$Instance" -s DNI_SYSADM -ou SYSOU -to $Timeout -c "list -ct Dnf% -lo nsl" >> dnfczmlc_CTAttrList
    rc=$?
  fi
  if [ $rc -eq 0 ]
  then
    dnicli -i "$Instance" -s DNI_SYSADM -ou SYSOU -to $Timeout -c "list -ct DNF% -lo nsl" >> dnfczmlc_CTAttrList
    rc=$?
  fi
  if [ $rc -eq 0 ]
  then
    dnicli -i "$Instance" -s DNI_SYSADM -ou SYSOU -to $Timeout -c "list -ct Dni% -attr Dnf% -lo nsl" >> dnfczmlc_CTAttrList
    rc=$?
  fi
  if [ $rc -eq 0 ]
  then
    dnicli -i "$Instance" -s DNI_SYSADM -ou SYSOU -to $Timeout -c "list -ct Dnq% -lo nsl" >> dnfczmlc_CTAttrList
    rc=$?
  fi
  if [ $rc -eq 0 ]
  then
    dnicli -i "$Instance" -s DNI_SYSADM -ou SYSOU -to $Timeout -c "list -ct Dni% -attr Dnq% -lo nsl" >> dnfczmlc_CTAttrList
    rc=$?
  fi
     # =================================================
     # Speical condition for SR2010 attr TestMsgDefSet
     # =================================================
  if [ $rc -eq 0 ]
  then
    dnicli -i "$Instance" -s DNI_SYSADM -ou SYSOU -to $Timeout -c "list -ct DniMsgDomain -attr % -lo nsl" >> dnfczmlc_CTAttrList
    rc=$?
  fi


  # ============================================================================
  # List all OU's containing CO's with CT's (DniConfigSVB, Dnf%, DNF% and DniMsgDomain) with their
  # Attributes and Types and store in dnfczmlc_COAttrList
  # ============================================================================
  if [ $rc -eq 0 ]
  then
    dnicli -i "$Instance" -s DNI_SYSADM -ou SYSOU -to $Timeout -c "list -ou % -ct Dnf% -lo ncoslv" > dnfczmlc_COAttrList
    rc=$?
  fi
# do the check for the list -ou % ... -lo ncoslv here
  if [ $rc -ne 0 ]
  then
    DnfLogMsg="CLI command does not run properly: dnicli -i $Instance -s DNI_SYSADM -ou SYSOU -to $Timeout -c list -ou % -ct Dnf% -lo ncoslv";
    DnfLogSeverity="DnfError";
    logmessage;
    DnfLogMsg="Script cannot run.";
    logmessage;
    DnfLogMsg="Please check the following:";
    logmessage;
    DnfLogMsg="  - Is the timeout interval for the CLI high enough?";
    logmessage;
    DnfLogMsg="  - Run the above command in the shell and see";
    logmessage;
    DnfLogMsg="    the error message issued by the CLI";
    logmessage;
  fi
  if [ $rc -eq 0 ]
  then
    dnicli -i "$Instance" -s DNI_SYSADM -ou SYSOU -to $Timeout -c "list -ou DNFSYSOU -ct DniConfigSVB -lo ncoslv" >> dnfczmlc_COAttrList
    rc=$?
  fi
  if [ $rc -eq 0 ]
  then
    dnicli -i "$Instance" -s DNI_SYSADM -ou SYSOU -to $Timeout -c "list -ou % -ct DNF% -lo ncoslv" >> dnfczmlc_COAttrList
    rc=$?
  fi
  if [ $rc -eq 0 ]
  then
    dnicli -i "$Instance" -s DNI_SYSADM -ou SYSOU -to $Timeout -c "list -ou % -ct Dnq% -lo ncoslv" >> dnfczmlc_COAttrList
    rc=$?
  fi
  if [ $rc -eq 0 ]
  then
    dnicli -i "$Instance" -s DNI_SYSADM -ou SYSOU -to $Timeout -c "list -ou % -ct Dni% -co Dnf% -lo ncoslv" >> dnfczmlc_COAttrList
    rc=$?
  fi
     # =================================================
     # Speical condition for SR2010 CO 
     # =================================================
  if [ $rc -eq 0 ]
  then
    dnicli -i "$Instance" -s DNI_SYSADM -ou SYSOU -to $Timeout -c "list -ou % -ct DniMsgDomain -co % -lo ncoslv" >> dnfczmlc_COAttrList
    rc=$?
  fi

  # ============================================================================
  # For dnfczcob / dnfczcoy Migration:
  # List all BOU's
  # ============================================================================
  if [ $rc -eq 0 ]
  then
    dnicli -i "$Instance" -s DNI_SYSADM -ou SYSOU -to $Timeout -c "list -ou %" > dnfczmlc_BOUList
    rc=$?
  fi

  if [ $rc -ne 0 ]
  then
    rm dnfczmlc_*;
    rc=16;
    DnfLogMsg="CLI command did not run properly.";
    DnfLogSeverity="DnfError";
    logmessage;
    callfooter;
    rc_max=$rc;
    exit $rc_max;
  fi


  # ============================================================================
  # For dnfczcob / dnfczcoy Migration:
  # List all CO's beginning with "Dnf%" ot "Dnq%" for CT's beginning either with
  # Dni%, DNF%, Dnf%, Dnq% or DniMsgDomain 
  # with their Attributes and Types and store in dnfczmlc_SpecCOAttrList
  # ============================================================================
  if [ $rc -eq 0 ]
  then
    dnicli -i "$Instance" -s DNI_SYSADM -ou SYSOU -to $Timeout -c "list -ou DNFSYSOU -ct DniConfigSVB -lo ncos" > dnfczmlc_SpecCOAttrList
    rc=$?
  fi
  
  if [ $rc -eq 0 ]
  then
    dnicli -i "$Instance" -s DNI_SYSADM -ou SYSOU -to $Timeout -c "list -ou % -ct Dni% -co Dnf% -lo ncos" >> dnfczmlc_SpecCOAttrList
    rc=$?
  fi

  if [ $rc -eq 0 ]
  then
    dnicli -i "$Instance" -s DNI_SYSADM -ou SYSOU -to $Timeout -c "list -ou % -ct Dni% -co DNF% -lo ncos" >> dnfczmlc_SpecCOAttrList
    rc=$?
  fi
  
  if [ $rc -eq 0 ]
  then
    dnicli -i "$Instance" -s DNI_SYSADM -ou SYSOU -to $Timeout -c "list -ou % -ct Dni% -co Dnq% -lo ncos" >> dnfczmlc_SpecCOAttrList
    rc=$?
  fi

  if [ $rc -eq 0 ]
  then
    dnicli -i "$Instance" -s DNI_SYSADM -ou SYSOU -to $Timeout -c "list -ou % -ct DNF% -lo ncos" >> dnfczmlc_SpecCOAttrList
    rc=$?
  fi


  if [ $rc -eq 0 ]
  then
    dnicli -i "$Instance" -s DNI_SYSADM -ou SYSOU -to $Timeout -c "list -ou % -ct Dnf% -lo ncos" >> dnfczmlc_SpecCOAttrList
    rc=$?
  fi
  
  if [ $rc -eq 0 ]
  then
    dnicli -i "$Instance" -s DNI_SYSADM -ou SYSOU -to $Timeout -c "list -ou % -ct Dnq% -lo ncos" >> dnfczmlc_SpecCOAttrList
    rc=$?
  fi
  
     # =================================================
     # Speical condition for SR2010 CO
     # =================================================
  if [ $rc -eq 0 ]
  then
    dnicli -i "$Instance" -s DNI_SYSADM -ou SYSOU -to $Timeout -c "list -ou % -ct DniMsgDomain -lo ncos" >> dnfczmlc_SpecCOAttrList
    rc=$?
  fi

  if [ $rc -ne 0 ]
  then
    rm dnfczmlc_*;
    rc=16;
    DnfLogMsg="CLI command did not run properly.";
    DnfLogSeverity="DnfError";
    logmessage;
    callfooter;
    rc_max=$rc;
    exit $rc_max;
  fi

}



# ============================================================================
# Migrate COS's
# ============================================================================
# Checks whether there still exist CO assignments
# if no files are generated 0, otherwise 1 is returned
# input Instance         (the instance name)
# input DnfLog           (name of the log file)
# input ESNcosfile       (name of CLI file to be used in order to identify the initial ESN COSs)
# input ESNctfile        (name of CLI file to be used in order to identify the initial ESN CTs)
# rc = 0 --> no files generated, 1 --> files generated

migrateCOSs()
{
  awk -f "$dirname/dnfczml1.awk" \
    logFile="$DnfLog" infologFile="$DnfInfoLog" instance="$Instance" \
    CreCOSCom="$tempCliCreCOSCom" CreCOSDep="$tempCliCreCOSDep" \
    RemCOSCom="$tempCliRemCOSCom" RemCOSDep="$tempCliRemCOSDep" \
    strBSUL="$strBSUL" strBSPC="$strBSPC" \
    x="0" "dnfczmlc_COSList" x="1" "dnfczmlc_COSCTCOList" x="2" "dnfczmlc_ExtCOSCTCOList" \
    x="3" "$temphisfile" x="4" "$tempctfile" x="5" "$tempcosfile"
}



# ============================================================================
# Migrate Roles, CT's and CO's
# ============================================================================
# Migrates the Role and CT ressources
# if no files are generated 0, otherwise 1 is returned
# input Instance         (the instance name)
# input DnfLog           (name of the log file)
# input ESNctfile        (name of CLI file to be used in order to identify the initial CTs)
# rc = 0 --> no files generated, 1 --> files generated
#
# replaced dnfczmlc_COList with dnfczmlc_COAttrList
migrateCTs()
{
  awk -f "$dirname/dnfczml2.awk" \
    logFile="$DnfLog" infologFile="$DnfInfoLog" instance="$Instance" \
    CreRoCom="$tempCliCreRoCom" CreRoApp="$tempCliCreRoApp" \
    RemRoCom="$tempCliRemRoCom" RemRoApp="$tempCliRemRoApp" \
    CreCTCom="$tempCliCreCTCom" CreCTDep="$tempCliCreCTDep" \
    RemCTCom="$tempCliRemCTCom" RemCTDep="$tempCliRemCTDep" \
    CreCOCom="$tempCliCreCOCom" CreCODep="$tempCliCreCODep" \
    RemCOCom="$tempCliRemCOCom" RemCODep="$tempCliRemCODep" \
    RemRoCom="$tempCliRemRoCom" RemRoApp="$tempCliRemRoApp" \
    ValCom="$tempCliValCom" ValDep="$tempCliValDep" \
    strBSUL="$strBSUL" strBSPC="$strBSPC" \
    x="0" "dnfczmlc_RoCTCOAttrList" x="1" "dnfczmlc_CTList" \
    x="2" "dnfczmlc_CTAttrList" x="3" "dnfczmlc_COAttrList" \
    x="4" "dnfczmlc_SpecCOAttrList" x="5" "dnfczmlc_BOUList" \
    x="6" "$temphisfile" x="7" "$tempctfile" x="8" "$temprolefile" x="9" "$tempcofile"
}



# ============================================================================
# Start main procedure
# ============================================================================

# Program name
PgmName="$0";
PgmName1="dnfczmlc";
# USS platform
DNI_USS="OS/390";
# AIX platform
DNI_AIX="AIX";
# Log file
DnfLog="$PgmName1"".log";
DnfInfoLog="$PgmName1"".inf";
# Default platform value
platform="";
# Return Code value
rc=0;
# Count used in name of CLI command files
cliCount=1;

# Delete all previously generated command files
touch dnfczmlc_temp;
rm dnfczmlc_*;

dirname=`dirname $PgmName`
command="$PgmName $*";

# Check and get the platform value
callplatform;

# Check the input parameters
callcheckparm $*;

# Write a heading line to the Log
callheader;


#----------------------------------------------------------#
# Generate CLI command files to Migrate ESN config entities
#----------------------------------------------------------#

checkSVBExists;
rc=$?
if [ rc -ne 0 ]
then
  DnfLogMsg="Database check failed.";
  DnfLogSeverity="DNFError";
  logmessage;
  callfooter;
  exit -1 
fi

# Check access to CT & COS file
echo "Checking access to the basic definition data files ...";

tempctfile="./""$PgmName1""_ct.cli";
tempcosfile="./""$PgmName1""_cos.cli";
tempcofile="./""$PgmName1""_co.cli";
temprolefile="./""$PgmName1""_ro.cli";
temphisfile="./""$PgmName1""_oldres.lst";

generateTempFile;
rc=$?
if [ rc -ne 0 ]
then
  DnfLogMsg="Generate temporary resource files.";
  DnfLogSeverity="DNFError";
  logmessage;
  callfooter;
  exit -1 
fi


echo "successful ...";
echo;


# Check function of CLI
echo "Checking the current configuration for instance $Instance ...";
checkCli;
rc=$?
if [ $rc -ne 0 ]
then
  callfooter;
  exit -1
fi


# Check database
ouToCheck="SYSOU"
checkOUExists;
rc=$?
if [ $rc -ne 0 ]
then
  callfooter;
  exit -1
fi

checkNotApprovedRo;
rc=$?
if [ $rc -ne 0 ]
then
  callfooter;
  exit -1
fi

checkNotDeployedCT;
rc=$?
if [ $rc -ne 0 ]
then
  callfooter;
  exit -1
fi

checkNotDeployedOU;
rc=$?
if [ $rc -ne 0 ]
then
  callfooter;
  exit -1
fi

checkNotDeployedCOS;
rc=$?
if [ $rc -ne 0 ]
then
  callfooter;
  exit -1
fi

queryCLI;
rc=$?
if [ $rc -ne 0 ]
then
  rc=16;
  DnfLogMsg="CLI command did not run properly.";
  DnfLogSeverity="DnfError";
  logmessage;
  callfooter;
  exit -1
fi

echo "successful ...";
echo;

echo "Generating the command files for migration ...";


# Initialize CLI files

tempCliRemRoCom="./""$PgmName1""_""$cliCount""_ua_rem_ro_com.cli";
tempCliRemRoApp="./""$PgmName1""_""$cliCount""_ua_rem_ro_app.cli";
cliCount=cliCount+1;

tempCliRemCOSCom="./""$PgmName1""_""$cliCount""_sa_rem_cos_com.cli";
tempCliRemCOSDep="./""$PgmName1""_""$cliCount""_sa_rem_cos_dep.cli";
cliCount=cliCount+1;

tempCliRemCOCom="./""$PgmName1""_""$cliCount""_sa_rem_co_com.cli";
tempCliRemCODep="./""$PgmName1""_""$cliCount""_sa_rem_co_dep.cli";
cliCount=cliCount+1;

tempCliRemCTCom="./""$PgmName1""_""$cliCount""_sa_rem_ct_com.cli";
tempCliRemCTDep="./""$PgmName1""_""$cliCount""_sa_rem_ct_dep.cli";
cliCount=cliCount+1;

tempCliCreCTCom="./""$PgmName1""_""$cliCount""_sa_cre_ct_com.cli";
tempCliCreCTDep="./""$PgmName1""_""$cliCount""_sa_cre_ct_dep.cli";
cliCount=cliCount+1;

tempCliCreCOCom="./""$PgmName1""_""$cliCount""_sa_cre_co_com.cli";
tempCliCreCODep="./""$PgmName1""_""$cliCount""_sa_cre_co_dep.cli";
cliCount=cliCount+1;

tempCliCreCOSCom="./""$PgmName1""_""$cliCount""_sa_cre_cos_com.cli";
tempCliCreCOSDep="./""$PgmName1""_""$cliCount""_sa_cre_cos_dep.cli";
cliCount=cliCount+1;

tempCliCreRoCom="./""$PgmName1""_""$cliCount""_ua_cre_ro_com.cli";
tempCliCreRoApp="./""$PgmName1""_""$cliCount""_ua_cre_ro_app.cli";

tempCliValCom="./""$PgmName1""_sa_val_com.cli";
tempCliValDep="./""$PgmName1""_sa_val_dep.cli";

# Initial Platform different Backslash string for awk gsub
platform=`uname`
if [ "$platform" = "$DNI_USS" ]
then
  strBSUL="\\\\\\\\\\\\\\\\_";
  strBSPC="\\\\\\\\\\\\\\\\%";
else
  strBSUL="\\\\\\\\_";
  strBSPC="\\\\\\\\%";
fi
# Migrate COS's & CT's

DnfLogSeverity="DnfInfo";
DnfLogMsg="Migrating the COS's:";
logmessage;

migrateCOSs;
rccos=$?

DnfLogSeverity="DnfInfo";
DnfLogMsg="Migrating the CT's:";
logmessage;

migrateCTs;
rcct=$?

rm $tempctfile;
rm $tempcosfile;
rm $tempcofile;
rm $temprolefile;
rm $temphisfile;
rm dnfczmlc_RoCTCOAttrList;
rm dnfczmlc_COSList;
rm dnfczmlc_COSCTCOList;
rm dnfczmlc_ExtCOSCTCOList;
rm dnfczmlc_CTList;
rm dnfczmlc_CTAttrList;
rm dnfczmlc_COAttrList;
rm dnfczmlc_SpecCOAttrList;
rm dnfczmlc_BOUList;

if [ $rcct -eq 16 ]
then
  rc=16;
  callfooter;
  exit -1
else
  if [ $rccos -eq 0 ] && [ $rcct -eq 0 ]
  then
    DnfLogSeverity="DnfInfo";
    DnfLogMsg="... no files generated (no migration necessary).";
    logmessage;
    echo "... no files generated (no migration necessary).";
  else
    if [ "$Dual" = "NO" ]
    then
      CliEntAll="./""$PgmName1""_2_sa_ent_all.cli";
      CliValAll="./""$PgmName1""_sa_val_all.cli";
      CliRemRoAll="./""$PgmName1""_1_ua_rem_ro_all.cli";
      CliCreRoAll="./""$PgmName1""_3_ua_cre_ro_all.cli";
      catFiles;
    fi
    echo "Following file(s) are generated and need to be executed for migration:";
    echo;
    ls -m1 "$PgmName1"_*;
    echo;
    echo "DnfInfo: Following files are generated and need to be executed for migration:" >> $DnfLog;
    ls -m1 "$PgmName1"_* >> $DnfLog;
    echo "Following files are generated and need to be executed for migration:" >> $DnfInfoLog;
    ls -m1 "$PgmName1"_* >> $DnfInfoLog;
  fi
fi

# Write a footer line to the Log
callfooter;

# end main procedure

